/*
 *      L I S T _ B e f o r e
 *
 *  This function is called when it is required
 *  to insert a new node before a specifed node.
 *
 *  We have actually only really implemented inserting
 *  nodes after a given node in the support routines,
 *  so we do the work to effectively change this into
 *  a List_After() type routine.
 *
 *
 * (c) Copyright 1996-1997 David J. Walker
 *     Email:  d.j.walker@x400.icl.co.uk
 *
 *     Permission to copy and/or distribute granted under the
 *     following conditions:
 *
 *     1). This notice must remain intact.
 *     2). The author is not responsible for the consequences of use
 *         this software, no matter how awful, even if they
 *         arise from defects in it.
 *     3). Altered version must not be represented as being the
 *         original software.
 *
 *
 *  AMENDMENT HISTORY
 *  ~~~~~~~~~~~~~~~~~
 *  22 May 96   DJW   - First version
 */

#include <liblist.h>
#include "liblistx.h"

int     LIST_Before (list_t list, node_t newnode, node_t oldnode)
{
    node_t   tempnode;
    listclass_t listclass;
    _listmaster_t listmaster;

    LIBDEBUG ("LIST_Before", \
            "'list' parameter is not a valid list pointer", \
            _LIST_ListCheck(list))
    LIBDEBUG ("LIST_Before", \
            "'newnode' parameter is not a valid member of this list type",\
            _LIST_NodeCheck(list,newnode))
    LIBDEBUG ("LIST_Before", \
            "'newnode is already a member of this list", \
             LIST_Index(list,newnode) == 0)

    listclass = list->listclass;
	listmaster = listclass->master;
    /*
     *  There is a special case of
     *  'oldnode' being NULL which
     *  means insert at front of list.
     */
    if (oldnode == NULL)
    {
        oldnode = listmaster->first(list);
    }

    if (oldnode == NULL)
    {
        /*
         *  Looks like an empty list!
         */

        tempnode = NULL;
    }
    else
    {
        tempnode = listmaster->previous(list, _LIST_REALNODE(list,oldnode));

        /*
         *  First see if this is an ordered list
         *  as then we need to do some checks on
         *  whether it is valid to insert at
         *  this point.
         */
        if (listclass->compare)
        {
            if (oldnode != NULL)
            {

                if (LIST_Compare(list,newnode,oldnode) > 0)
                {
                    return (LIST_ERROR_SEQUENCE);
                }
                if (tempnode != NULL)
                {
                    if (LIST_Compare(list,newnode,tempnode) < 0)
                    {
                        return (LIST_ERROR_SEQUENCE);
                    }
                }
            }
        }
        if (tempnode != NULL)
		{
            tempnode = _LIST_REALNODE(list,tempnode);
        }
    }
    newnode = _LIST_REALNODE(list,newnode);
    /*
     *  either we passed checks or list is unordered.
     *  Call type specific support routine to add node.
     */
    return (listmaster->after(list,newnode,tempnode));
}

