/*
 *      _ L I S T _ L i f o R e m o v e
 *
 *  This is used to remove a node from a 
 *  singled linked list that DOES NOT have
 *  a pointer to the last node (LIFO).
 *
 *
 * (c) Copyright 1996-1997 David J. Walker
 *     Email:  d.j.walker@x400.icl.co.uk
 *
 *     Permission to copy and/or distribute granted under the
 *     following conditions:
 *
 *     1). This notice must remain intact.
 *     2). The author is not responsible for the consequences of use
 *         this software, no matter how awful, even if they
 *         arise from defects in it.
 *     3). Altered version must not be represented as being the
 *         original software.
 *
 *
 *  AMENDMENT HISTORY
 *  ~~~~~~~~~~~~~~~~~
 *  23 May 96   DJW   - First version
 *
 *  28 May 96   DJW   - Updated 'remove' to allow for the user providing
 *                      a member to remove who is not on the list!
 */


#include <liblist.h>
#include "liblistx.h"


int     _LIST_LifoRemove (list_t list, node_t node)

#define _list   ((_lifoh_t)list)
#define usernode  ((_lifo_t)node)
{
    _lifoh_t  lifolist = (_lifoh_t)list;
	_lifo_t lifonode = (_lifo_t)node;
	_lifo_t listnode;

    if (lifonode == NULL)
	{
		return (LIST_ERROR_BADNODE);
	}
    /*
     *  If this is the first node, then
     *  we have an easy life!
     */
    if (lifonode == (_lifo_t)list->nodes)
    {
        lifolist->nodes = lifonode->next;
    }
    else
    {
        /*
         *  We need to scan the whole list in sequence
         */
        for (listnode = lifolist->nodes ; listnode != NULL ; listnode = listnode->next)
        {
            if (listnode->next == lifonode)
            {
                break;
            }
        }
        /*

         *  We had better check that we found it!
         *  The user could give us a member not attached
         *  (or a NULL value which has the same effect).
         */
        if (listnode == NULL)
        {
            return LIST_ERROR_NOTFOUND;
        }
        listnode->next = lifonode->next;
    }
    lifonode->next = NULL;
    return (LIST_ERROR_NONE);
}

