/*
 *      _ L I S T _ L i s t A d d
 *
 *  This is an internal support routine
 *  to insert a node at the correct point in
 *  an ordered list.   Before we get here we
 *  can assume that the list has already been
 *  checked to see if it is ordered.
 *
 *
 * (c) Copyright 1996-1997 David J. Walker
 *     Email:  d.j.walker@x400.icl.co.uk
 *
 *     Permission to copy and/or distribute granted under the
 *     following conditions:
 *
 *     1). This notice must remain intact.
 *     2). The author is not responsible for the consequences of use
 *         this software, no matter how awful, even if they
 *         arise from defects in it.
 *     3). Altered version must not be represented as being the
 *         original software.
 *
 *
 *  AMENDMENT HISTORY
 *  ~~~~~~~~~~~~~~~~~
 *  18 May 96   DJW   - First version
 */


#include <liblist.h>
#include "liblistx.h"


extern
int     _LIST_ListAdd (list_t list, node_t newnode)
{
	_list1h_t singlelist = (_list1h_t)list;
	listclass_t listclass;
    _list1_t nextnode;
	_list1_t prevnode;

    LIBASSERT (newnode != NULL)
    /*
     *  We need to scan the whole list in sequence
     */
    for (prevnode = NULL, nextnode = singlelist->nodes, listclass=singlelist->listclass ; 
				nextnode != NULL ; 
						prevnode = nextnode, nextnode = nextnode->next)
    {
        /*
         *  If the next node would be too far,
         *  then we must insert after the previous
         *  node (if any) in the list.
         */
        if (listclass->compare (_LIST_USERNODE(singlelist, newnode), 
                                _LIST_USERNODE(singlelist,nextnode)) < 0)
        {
            break;
        }
    }
    return (listclass->master->after((list_t)singlelist, newnode, (node_t)prevnode));
}

