/*
 *      _ L I S T _ N o d e C h e c k
 *
 *  This function is an internal support routine that
 *  is used in the debug version of the library to
 *  check that a node appears to be valid as a member
 *  of the given list.
 *
 *  NOTE.   This check depends on all node types
 *          having the check field in the same position
 *          (i.e. as the first field).
 *
 * (c) Copyright 1996-1997 David J. Walker
 *     Email:  d.j.walker@x400.icl.co.uk
 *
 *     Permission to copy and/or distribute granted under the
 *     following conditions:
 *
 *     1). This notice must remain intact.
 *     2). The author is not responsible for the consequences of use
 *         this software, no matter how awful, even if they
 *         arise from defects in it.
 *     3). Altered version must not be represented as being the
 *         original software.
 *
 *
 *  AMENDMENT HISTORY
 *  ~~~~~~~~~~~~~~~~~
 *  25 May 96   DJW   - First version.
 *
 *  10 July 96  DJW   - Reworked to be more comprehensive, and correctly
 *						handle the case of embedded lists.
 */


#include <liblist.h>
#include "liblistx.h"


extern
int    _LIST_NodeCheck     (list_t list, node_t node)
{
#ifdef DEBUG
	listclass_t  listclass;
	listclass_t  nodeclass;
    _nodecheck_t realnode;
#endif /* DEBUG */

	/*
	 * It is an error if the node
	 * value supplied is NULL.
	 */
    if (node == NULL)
    {
        return 0;
    }
#ifdef DEBUG
	/*
	 *	For embedded lists we need to move down
	 *	a level to the real list type.
	 */
    if (LIST_Embedded(list))
	{
		_embedh_t  embedlist = (_embedh_t)list;
		list = embedlist->self;
		LIBASSERT (_LIST_ListCheck (list))
	}
	/*
	 *	For hash lists, we need to get the type
	 *	of the attached list if there is one.  If
	 *	this is itself an attached list then we
	 *	will need to recurse down.
	 */
	while (LIST_Type(list) == LIST_TYPE_HASH)
	{
		if (((_hashh_t)list)->clonelist == NULL)
		{
			break;
		}
		list = ((_hashh_t)list)->clonelist;
	}

	/*
	 * Convert User view of the node
     * to an internal library view,
	 * and then check the simple case.
	 */
    realnode = (_nodecheck_t)_LIST_REALNODE(list,node);
	LIBASSERT (realnode->list != NULL)
	LIBASSERT (realnode->list->listclass != NULL)
	LIBASSERT (list->listclass != NULL)
	nodeclass = realnode->list->listclass;
	listclass = list->listclass;

	/*
	 *	Check fundemental base classes match
	 */
    if (listclass->master->listtype != nodeclass->master->listtype)
	{
		return 0;
	}
	/*
	 *	Check key sizes agree
	 */
	if (listclass->sizenode != nodeclass->sizenode)
	{
		return 0;
	}
	if (listclass->sizeuser != nodeclass->sizeuser)
	{
		return 0;
	}
	if (listclass->sizelist != nodeclass->sizelist)
	{
		return 0;
	}
#endif /* DEBUG */

#ifndef EPOC
    (void)list;     /* lets stop compiler warnings! */
#endif /* EPOC */

    return (1);
}


