/*
 *      T r a c k . c
 *
 *  These are internal support functions for the LIBLIST library.
 *
 *  They are used to protect against trying to delete a list
 *  either does not exist, or ha already been deleted.
 *
 *
 * (c) Copyright 1996-1997 David J. Walker
 *     Email:  d.j.walker@x400.icl.co.uk
 *
 *     Permission to copy and/or distribute granted under the
 *     following conditions:
 *
 *     1). This notice must remain intact.
 *     2). The author is not responsible for the consequences of use
 *         this software, no matter how awful, even if they
 *         arise from defects in it.
 *     3). Altered version must not be represented as being the
 *         original software.
 *
 *
 *  AMENDMENT HISTORY
 *  ~~~~~~~~~~~~~~~~~
 *  18 May 96   DJW   - First version
 */

#include <liblist.h>
#include "liblistx.h"

typedef struct tracking {
        list_t  list;
        } * tracking_t;

static  list_t  TrackingList = NULL;

/*===================================================== TRACK_NODEINIT */
static
int     Tracking_NodeInit (node_t node, va_list ap)
/*      ~~~~~~~~~~~~~~~~~
 *  This is the node initialisation function.
 *---------------------------------------------------------------------*/
{
    ((tracking_t)node)->list = va_arg (ap, list_t);
    return LIST_ERROR_NONE;
}


/*======================================================= TRACKING_FIND */
static
int     Tracking_Find (node_t node, va_list ap)
/*      ~~~~~~~~~~~~~
 *----------------------------------------------------------------------*/
{
    list_t  list = va_arg(ap, list_t);

    if (((tracking_t)node)->list == list)
    {
        return 0;
    }
    return ((tracking_t)node)->list < list ? -1 : +1;
}


/*========================================================= TRACK_ADD */
extern
void    _LIST_TrackAdd  (list_t list)
/*      ~~~~~~~~~~~~~~
 *  Add a list entry to those that are currently
 *  known to be in existence.
 *--------------------------------------------------------------------*/
{
    if (TrackingList == NULL)
    {
        TrackingList = LIST_Create (LIST_CLASS_LIFO,
                                    sizeof (struct tracking),
                                    Tracking_NodeInit,
                                    NULL,
                                    NULL);
    }
    (void) LIST_NewAdd (TrackingList, list);
    return;
}

/*======================================================= TRACK_DELETE */
extern
int     _LIST_TrackDelete (list_t list)
/*      ~~~~~~~~~~~~~~~~~
 *  Delete the given node from the list of those being tracked.
 *--------------------------------------------------------------------*/
{
    node_t  node;

    node = (node_t)LIST_Find (TrackingList, Tracking_Find, list);
    if (node == NULL)
    {
        return LIST_ERROR_NOTFOUND;
    }
    (void) LIST_Delete (TrackingList, node);
    return LIST_ERROR_NONE;
}
